//-------------------------------------------------------------------
// Copyright (c) 2011, Jeff Preshing
// http://preshing.com/20110811/xkcd-password-generator
// All rights reserved.
//
// Some parts based on http://www.mytsoftware.com/dailyproject/PassGen/entropy.js, copyright 2003 David Finch.
// 
// Released under the Modified BSD License:
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the <organization> nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//-------------------------------------------------------------------

// A SHA-1 hash generated on the server which is unique for each visit
var xkcd_pw_gen_server_hash = "6fec8d5d85c0235235e78b0374cce7c967bba2ef";

var xkcd_pw_gen_wordlist;

var DSLScript  = document.createElement("script");
DSLScript.src  = "js/french.min.js";
DSLScript.type = "text/javascript";
document.body.appendChild(DSLScript);

// Get some entropy from the system clock
function xkcd_pw_gen_time_ent()
{
    var d = 1 * new Date();
    var i = 0;
    while (1 * new Date() == d)
        i++; // Measure iterations until next tick
    return "" + d + i;
}

// Return a pseudorandom array of four 32-bit integers
function xkcd_pw_gen_create_hash()
{
    // Entropy string built in a manner inspired by David Finch
    var entropy = xkcd_pw_gen_server_hash + xkcd_pw_gen_time_ent();
    entropy += navigator.userAgent + Math.random() + Math.random() + screen.width + screen.height;
    if (document.all)
        entropy = entropy + document.body.clientWidth + document.body.clientHeight + document.body.scrollWidth + document.body.scrollHeight;
    else
        entropy = entropy + window.innerWidth + window.innerHeight + window.width + window.height;
    entropy += xkcd_pw_gen_time_ent();
        
    // Hash and convert to 32-bit integers
    var hexString = hex_sha1(entropy); // from sha1-min.js
    var result = [];
    for (var i = 0; i < 32; i += 8)
    {
        result.push(parseInt(hexString.substr(i, 8), 16));
    }
    return result;
}

// Diceware : https://www.rempe.us/diceware/#french
// And https://github.com/grempe/diceware
function calcCrackTime (nWords, guessesPerSec) {
    var keySpace,
      halfKeySpace,
      seconds,
      minutes,
      hours,
      days,
      years,
      millenia,
      humanLifetimes,
      universeLifetimes,
      avgHumanLifespanInYears,
      ageOfUniverseInYears;
    
    Big.E_NEG = -100;
    Big.E_POS = 100;
    
    // https://en.wikipedia.org/wiki/Life_expectancy
    avgHumanLifespanInYears = new Big(67.2);

    // https://en.wikipedia.org/wiki/Age_of_the_universe
    ageOfUniverseInYears = new Big(13798000000);

    // https://xkcd.com/936/
    // https://security.stackexchange.com/questions/62832/is-the-oft-cited-xkcd-scheme-no-longer-good-advice/62881#62881
    // https://hashcat.net/forum/thread-2580.html
    // Dictionnaire français fondamental : http://cursus.edu/article/7880/francais-fondamental-les-mots-indispensables-notre#.V74blyiLTDc
    keySpace = new Big(Math.pow(3726, nWords));

    // Divide the keySpace in half. On average it is expected that an
    // exhaustive search of only half the keySpace will result in success.
    halfKeySpace = keySpace.div(2);

    // "Assume that your adversary is capable of a trillion guesses per second" - Snowden
    // http://www.nytimes.com/2013/08/18/magazine/laura-poitras-snowden.html?pagewanted=all&_r=0
    guessesPerSec = new Big(1000000000000);
    
    seconds = halfKeySpace.div(guessesPerSec);
    minutes = seconds.div(60);
    hours = minutes.div(60);
    days = hours.div(24);
    years = days.div(365);
    millenia = years.div(1000);
    humanLifetimes = years.div(avgHumanLifespanInYears);
    universeLifetimes = years.div(ageOfUniverseInYears);

    // All values returned are of 'Big' type.
    // See : https://mikemcl.github.io/big.js/
    return {nWords: nWords, guessesPerSec: guessesPerSec, keySpace: keySpace, halfKeySpace: halfKeySpace, seconds: seconds, hours: hours, minutes: minutes, days: days, years: years, avgHumanLifespanInYears: avgHumanLifespanInYears, humanLifetimes: humanLifetimes, millenia: millenia, ageOfUniverseInYears: ageOfUniverseInYears, universeLifetimes: universeLifetimes};
}

// Formate un chiffre avec 'decimal' chiffres après la virgule et un separateur de millers
// http://www.toutjavascript.com/source/formater.html
function format(valeur,decimal,separateur) {
	var deci = new Big(Math.round( Math.pow(10,decimal)*(Math.abs(valeur)-Math.floor(Math.abs(valeur))))); 
	var val = new Big(Math.floor(Math.abs(valeur)));
	if ((decimal==0)||(deci==Math.pow(10,decimal))) {val=Math.floor(Math.abs(valeur)); deci=0;}
	var val_format=val+"";
	var nb=val_format.length;
	for (var i=1;i<10;i++) {
		if (val>=Math.pow(10,(3*i))) {
			val_format=val_format.substring(0,nb-(3*i))+separateur+val_format.substring(nb-(3*i));
		}
	}
	if (decimal>0) {
		var decim=""; 
		for (var j=0;j<(decimal-deci.toString().length);j++) {decim+="0";}
		deci=decim+deci.toString();
		val_format=val_format+","+deci;
	}
	if (parseFloat(valeur)<0) {val_format="-"+val_format;}
	return val_format;
}

// Formate le separateur de milliers
// http://hilog.co/fonction-javascript-de-separateur-de-milliers-63.html
function separateurMillier(num) {
	return ("" + num).replace(/(\d)(?=(\d\d\d)+(?!\d))/g, function($1) { return $1 + " " });
}

// Generate a new words passphrase and update the document
function xkcd_pw_gen(form_element)
{
    numWords = form_element.id;
    var hash = xkcd_pw_gen_create_hash();
    var choices = [];
    for (var w = 0; w < numWords; w++)
    {
        var jsRandom = Math.floor(Math.random() * 0x100000000);
        var index = ((jsRandom ^ hash[w]) + 0x100000000) % xkcd_pw_gen_wordlist.length;
        choices.push(xkcd_pw_gen_wordlist[index]);
    }
    resultSpan = document.getElementById("xkcd_pw_gen_result");
    resultSpan.innerText = resultSpan.textContent = choices.join(" ");
    
    // Display the block crack time results
    document.getElementById("statEntropie").style.display = "block";
    
    // Display crack time results (Diceware https://github.com/grempe/diceware)
    var crackTimeResults = calcCrackTime(numWords);
    document.getElementById('totalWords').innerHTML = numWords;
    document.getElementById('totalEntropy').innerHTML = format(numWords*11.86, 2, " ");
    document.getElementById('crackTimeResultsGuessesPerSecond').innerHTML = format(crackTimeResults.guessesPerSec, 0, "&nbsp;");
    document.getElementById('crackTimeResultsHalfKeySpace').innerHTML = separateurMillier(crackTimeResults.halfKeySpace);
    /* document.getElementById('crackTimeResultsSeconds').innerHTML = format(crackTimeResults.seconds, 2, "&nbsp;"); */
    document.getElementById('crackTimeResultsMinutes').innerHTML = format(crackTimeResults.minutes, 2, "&nbsp;");
    document.getElementById('crackTimeResultsHours').innerHTML = format(crackTimeResults.hours, 2, "&nbsp;");
    document.getElementById('crackTimeResultsDays').innerHTML = format(crackTimeResults.days, 2, "&nbsp;");
    document.getElementById('crackTimeResultsYears').innerHTML = format(crackTimeResults.years, 2, "&nbsp;");
    /* document.getElementById('crackTimeResultsHumanLifetimes').innerHTML = format(crackTimeResults.humanLifetimes, 9, "&nbsp;"); */
    /* document.getElementById('crackTimeResultsMillenia').innerHTML = format(crackTimeResults.millenia, 10, "&nbsp;"); */
    /* document.getElementById('crackTimeResultsUniverseLifetimes').innerHTML = format(crackTimeResults.universeLifetimes, 11, "&nbsp;"); */
}
xkcd_pw_gen();
